from moviepy.editor import VideoClip
from PIL import Image, ImageDraw, ImageColor, ImageFont
import numpy
import math

colors = ["#66c2a5", "#fc8d62", "#8da0cb", "#e78ac3", "#a6d854", "#ffd92f"]
background = "#2A2826"
white = "#E6E6E6"

number = "1"

w, h = 500, 500
antialias = 4
fps = 30
dur = 3

def make_frame(t):
    im = Image.new("RGB", (w * antialias, h * antialias), background)
    draw = ImageDraw.Draw(im)

    delta_ang = math.pi / 10
    circle_r = 75 * antialias
    circle_width = 5 * antialias
    start_shift = math.pi * 2 * circle_r / 6 * (t / dur)

    for i in range(-10, 100):
        cur_ang = i * delta_ang
        r = cur_ang * circle_r / 6 + start_shift
        x = math.cos(cur_ang) * r + im.size[0] / 2
        y = math.sin(cur_ang) * r + im.size[1] / 2

        draw.ellipse([x - circle_r, y - circle_r, x + circle_r, y + circle_r], fill=white)
        draw.ellipse([x - circle_r + circle_width, y - circle_r + circle_width, x + circle_r - circle_width, y + circle_r - circle_width], fill=background)

    big_circle_r = w * antialias * 0.8 / 2

    mask = Image.new("RGBA", (w * antialias, h * antialias), background)
    draw_mask = ImageDraw.Draw(mask)

    cx = w * antialias / 2
    cy = h * antialias / 2

    draw_mask.ellipse([cx - big_circle_r, cy - big_circle_r, cx + big_circle_r, cy + big_circle_r], fill=white)

    transparent_mask = Image.new("L", mask.size, color=255)
    draw_transparent_mask = ImageDraw.Draw(transparent_mask)
    draw_transparent_mask.ellipse([cx - big_circle_r + circle_width, cy - big_circle_r + circle_width, cx + big_circle_r - circle_width, cy + big_circle_r - circle_width], fill=0)

    mask.putalpha(transparent_mask)

    #mask.show()

    im.paste(mask, (0, 0), mask=mask)

    font_size = 30 * antialias
    font = ImageFont.truetype("Roboto-Medium.ttf", font_size)
    """
    text = u"Спецкурс по рисованию gif-ок"
    textsize = draw.textsize(text, font)
    draw.text(((w * antialias - textsize[0]) / 2, 10), text, white, font=font)
    """
    #draw.text((10, 10), "Test", white, font=font)

    im = im.resize([w, h], Image.LANCZOS)

    if (t == 0):
        im.save("frame" + number + ".png")

    return numpy.asarray(im)

clip = VideoClip(make_frame, duration=dur)

clip.write_gif("img" + number + ".gif", fps=fps)