from moviepy.editor import VideoClip
from PIL import Image, ImageDraw, ImageColor, ImageFont
import numpy
import math

colors = ["#66c2a5", "#fc8d62", "#8da0cb", "#e78ac3", "#a6d854", "#ffd92f"]
background = "#2A2826"
white = "#E6E6E6"

number = "2"

w, h = 500, 500
antialias = 4
fps = 30
dur = 90

def make_frame(t):
    steps = 12
    im = Image.new("RGB", (w * antialias, h * antialias), background)
    draw = ImageDraw.Draw(im)
    
    cx = w * antialias / 2
    cy = h * antialias / 2
    circle_r = 5 * antialias
    amount = 12
    scale = w * antialias / 2 / (amount + 1)

    draw.ellipse([cx - circle_r, cy - circle_r, cx + circle_r, cy + circle_r], fill=white)

    for i in range(1, amount + 1):
        r = i * scale
        ang = (t / dur * steps) * (amount - i + 1) * math.pi * 2
        
        cur_x = cx + math.cos(ang) * r
        cur_y = cy + math.sin(ang) * r

        draw.ellipse([cur_x - circle_r, cur_y - circle_r, cur_x + circle_r, cur_y + circle_r], fill=colors[(i - 1) % len(colors)])
                
    im = im.resize([w, h], Image.LANCZOS)

    if (t == 0):
        im.save("frame" + number + ".png")

    return numpy.asarray(im)

clip = VideoClip(make_frame, duration=dur)

clip.write_gif("img" + number + ".gif", fps=fps)