from moviepy.editor import VideoClip
from PIL import Image, ImageDraw, ImageColor, ImageFont
import numpy
import math

colors = ["#66c2a5", "#fc8d62", "#8da0cb", "#e78ac3", "#a6d854", "#ffd92f"]
background = "#2A2826"
white = "#E6E6E6"

number = "4"

w, h = 500, 500
antialias = 4
fps = 30
dur = 1
                              
def draw_hex(draw, x, y, r):
    points = []
    for i in range(6):
        cur_ang = math.pi / 2 + i * math.pi / 3
        points.append((x + math.cos(cur_ang) * r, y + math.sin(cur_ang) * r))
    draw.polygon(points, fill=white)

def make_frame(t):
    im = Image.new("RGB", (w * antialias, h * antialias), background)
    draw = ImageDraw.Draw(im)
    
    cx = w * antialias / 2
    cy = h * antialias / 2
    hex_size = w * antialias / 6
    hex_r = hex_size / 2 * 0.95

    real_time = (-math.cos(t / dur * math.pi) + 1) / 2
    #real_time = t / dur

    draw_hex(draw, cx, cy, hex_r)
    for i in range(6):
        cur_ang = i * math.pi / 3
        r = (real_time + 1) * hex_size
        cur_x = cx + math.cos(cur_ang) * r
        cur_y = cy + math.sin(cur_ang) * r
        draw_hex(draw, cur_x, cur_y, hex_r)

        r = (real_time * 0.8 + 2) * hex_size
        cur_x = cx + math.cos(cur_ang) * r
        cur_y = cy + math.sin(cur_ang) * r
        draw_hex(draw, cur_x, cur_y, hex_r * (1 - real_time))

        r = (real_time * 0.5 + 0.5) * hex_size
        cur_x = cx + math.cos(cur_ang) * r
        cur_y = cy + math.sin(cur_ang) * r
        draw_hex(draw, cur_x, cur_y, hex_r * real_time)
        
        r = hex_size
        coef = real_time * 0.4 + 1
        cur_x = cx + (math.cos(cur_ang) * r + math.cos(cur_ang + math.pi / 3) * r) * coef
        cur_y = cx + (math.sin(cur_ang) * r + math.sin(cur_ang + math.pi / 3) * r) * coef
        draw_hex(draw, cur_x, cur_y, hex_r * (1 - real_time))

        coef = real_time * 0.5 + 0.5
        cur_x = cx + (math.cos(cur_ang) * r + math.cos(cur_ang + math.pi / 3) * r) * coef
        cur_y = cx + (math.sin(cur_ang) * r + math.sin(cur_ang + math.pi / 3) * r) * coef
        draw_hex(draw, cur_x, cur_y, hex_r * real_time)


    im = im.resize([w, h], Image.LANCZOS)

    if (t == 0):
        im.save("frame" + number + ".png")

    return numpy.asarray(im)

clip = VideoClip(make_frame, duration=dur)

clip.write_gif("img" + number + ".gif", fps=fps)