from moviepy.editor import VideoClip
from PIL import Image, ImageDraw, ImageColor, ImageFont
import numpy
import math

colors = ["#66c2a5", "#fc8d62", "#8da0cb", "#e78ac3", "#a6d854", "#ffd92f"]
background = "#2A2826"
white = "#E6E6E6"

number = "6"

w, h = 500, 500
antialias = 4
fps = 30
dur = 3

def draw_circle(draw, x, y, r):
    draw.ellipse([x - r, y - r, x + r, y + r], fill=white)

def draw_ellipse(im, bound, width=1, outline=white):
    mask_top = int(bound[0] - width)
    mask_left = int(bound[1] - width)
    mask = Image.new("L", [int(bound[2] - bound[0] + width * 2), int(bound[3] - bound[1] + width * 2)], 0)
    mask_draw = ImageDraw.Draw(mask)
    
    for offset, color in [[width * 0.5, 255], [-width * 0.5, 0]]:
        top, left = [num - offset for num in bound[:2]]
        bottom, right = [num + offset for num in bound[2:]]
        top -= mask_top
        bottom -= mask_top
        left -= mask_left
        right -= mask_left
        mask_draw.ellipse([top, left, bottom, right], fill=color)

    im.paste(outline, (mask_top, mask_left), mask=mask)

def make_frame(t):
    im = Image.new("RGB", (w * antialias, h * antialias), background)
    draw = ImageDraw.Draw(im)
    
    amount = 10

    real_time = t / dur

    cx = w * antialias / 2
    cy = h * antialias / 2
    delta = w / (amount * 2 + 1) * antialias
    tot_high = 30 * antialias
    slope_ang = math.pi / 2 * 0.8

    for i in range(amount):
        r = (0.5 + i) * delta
        z = math.sin(r + real_time * math.pi * 2) * tot_high

        cur_x = cx
        cur_y = cy + z

        #draw.ellipse([cur_x - r, cur_y - r * math.cos(slope_ang), cur_x + r, cur_y + r * math.cos(slope_ang)])
        draw_ellipse(im, [cur_x - r, cur_y - r * math.cos(slope_ang), cur_x + r, cur_y + r * math.cos(slope_ang)], outline=white, width=2*antialias)
    
    im = im.resize([w, h], Image.LANCZOS)

    if (t == 0):
        im.save("frame" + number + ".png")

    return numpy.asarray(im)

clip = VideoClip(make_frame, duration=dur)

clip.write_gif("img" + number + ".gif", fps=fps)