from moviepy.editor import VideoClip
from PIL import Image, ImageDraw, ImageColor, ImageFont
import numpy
import math

colors = ["#ffd92f", "#fc8d62", "#8da0cb", "#66c2a5", "#e78ac3", "#a6d854"]
background = "#2A2826"
white = "#E6E6E6"

w, h = 500, 500
antialias = 4
fps = 30
dur = 2

deltas = [[1, 1], [1, -1], [-1, -1], [-1, 1]]

def draw_cube(im, draw, x, y, z):
    #print(x, y, z)
    cube_size = 30 * antialias

    x *= cube_size
    y *= cube_size
    z *= cube_size

    cx = w * antialias / 2
    cy = h * antialias / 2

    coords = [x, y, z]

    for i in range(0, 3):
        new_coords = [0, 0, 0]
        new_coords[i] = coords[i] - cube_size / 2
        arr_posses = []
        for delta in deltas:
            c = 0
            for j in range(3):
                if (j != i):
                    new_coords[j] = coords[j] + cube_size / 2 * delta[c]
                    c += 1
            nx, ny, nz = new_coords
            cur_x = cx - nx * math.sin(math.pi / 3) + ny * math.sin(math.pi / 3)
            cur_y = cy + nz - (nx * math.cos(math.pi / 3) + ny * math.cos(math.pi / 3))
            arr_posses.append((cur_x, cur_y))

        draw.polygon(arr_posses, fill=colors[i])


def find_coeffs(pa, pb):
    matrix = []
    for p1, p2 in zip(pa, pb):
        matrix.append([p1[0], p1[1], 1, 0, 0, 0, -p2[0]*p1[0], -p2[0]*p1[1]])
        matrix.append([0, 0, 0, p1[0], p1[1], 1, -p2[1]*p1[0], -p2[1]*p1[1]])

    A = numpy.matrix(matrix, dtype=numpy.float)
    B = numpy.array(pb).reshape(8)

    res = numpy.dot(numpy.linalg.inv(A.T * A) * A.T, B)
    return numpy.array(res).reshape(8)


def make_frame(t):
    im = Image.new("RGB", (w * antialias, h * antialias), background)
    draw = ImageDraw.Draw(im)

    real_time = t / dur

    scale_x = 1
    scale_y = 1
    scale_z = 1

    if (real_time < 1 / 6):
        scale_x = real_time * 6 + 1
    elif (real_time < 2 / 6):
        scale_x = 2
        scale_y = (real_time - 1 / 6) * 6 + 1
    elif (real_time < 3 / 6):
        scale_x = scale_y = 2
        scale_z = (real_time - 2 / 6) * 6 + 1
    elif (real_time < 4 / 6):
        scale_y = scale_z = 2
        scale_x = 2 - (real_time - 3 / 6) * 6
    elif (real_time < 5 / 6):
        scale_z = 2
        scale_y = 2 - (real_time - 4 / 6) * 6
    else:
        scale_z = 2 - (real_time - 5 / 6) * 6
    
    scale_x = (scale_x - 1) * 1.5 + 1
    scale_y = (scale_y - 1) * 1.5 + 1
    scale_z = (scale_z - 1) * 1.5 + 1

    for i in range(1, -2, -1):
        for j in range(1, -2, -1):
            for k in range(1, -2, -1):
                draw_cube(im, draw, i * scale_x, j * scale_y, k * scale_z)
    #draw_cube(draw, -scale_x, scale_y, scale_z)
    im = im.resize((w, h), Image.LANCZOS)

    return numpy.asarray(im)


clip = VideoClip(make_frame, duration=dur)
clip.write_gif("img2.gif", fps=fps)

"""
im = Image.new("RGB", (w * antialias, h * antialias), background)
draw = ImageDraw.Draw(im)
draw_cube(draw, 0, 0, 0)
draw_cube(draw, 2, 2, 0)
draw_cube(draw, -1.5, -1.5, 0)
im.save("5.png")
"""